#!/usr/bin/python

import sys, struct
 
class WiiLZSS:
    TYPE_LZSS = 0x11
    def __init__(self, file, offset):
        self.file = file
        self.offset = offset
 
        self.file.seek(self.offset)
 
        hdr = struct.unpack("<I",self.file.read(4))[0]
        self.uncompressed_length = hdr>>8
        self.compression_type = hdr & 0xFF
 
        if self.compression_type != self.TYPE_LZSS:
            raise ValueError("Unsupported compression method %d"%self.compression_type)
 
    def uncompress(self):
        dout = ""
 
        self.file.seek(self.offset + 0x4)
 
        while len(dout) < self.uncompressed_length:
            flags = struct.unpack("<B",self.file.read(1))[0]
 
            for i in range(8):
                # logic and commments from DSDecmp
                if flags & 0x80:
                    b1 = struct.unpack("<B",self.file.read(1))[0]
                    if (b1>>4) == 0:
                        # ab cd ef
                        # =>
                        # len = abc + 0x11 = bc + 0x11
                        # disp = def

                        bs = struct.unpack("<2B",self.file.read(2))
                        num = ((b1 << 4) | (bs[0] >> 4))
                        num += 0x11
                        disp = ((bs[0]<<8)&0xF00) | bs[1]
                    elif (b1>>4) == 1:
                        # ab cd ef gh
                        # =>
                        # len = bcde + 0x111
                        # disp = fgh

                        bs = struct.unpack("<3B",self.file.read(3))
                        num = ((b1<<12)&0xF000) | (bs[0]<<4) | ((bs[1]>>4)&0xF)
                        num += 0x111
                        disp = ((bs[1]<<8)&0xF00) | bs[2]
                    else:
                        # ab cd
                        # =>
                        # len = a + threshold = a + 1
                        # disp = bcd

                        bs = struct.unpack("<B",self.file.read(1))
                        num = (b1>>4)
                        num += 1
                        disp = ((b1<<8)&0xF00) | bs[0]
                    ptr = len(dout) - disp - 1
                    for i in range(num):
                        dout += dout[ptr]
                        ptr+=1
                        if len(dout) >= self.uncompressed_length:
                            break
                else:
                    dout += self.file.read(1)
                flags <<= 1
                if len(dout) >= self.uncompressed_length:
                    break
 
        self.data = dout
        return self.data
 
f = open(sys.argv[1])
 
hdr = f.read(4)
if hdr != "LZSS":
    f.seek(0)
 
unc = WiiLZSS(f, f.tell())
 
du = unc.uncompress()
 
f2 = open(sys.argv[2],"w")
f2.write(du)
f2.close()
